use ExtUtils::MakeMaker;
use 5.006;
use strict;
use warnings;
use Config;

use File::Spec::Functions; # export the functions into &main namespace
use File::Basename;

# Grab out any custom cmdline args
my %args = (
	pkg_name => 'eidetic',
	name     => 'eidetic',
	DESTDIR  => undef,
);

my @pass_args;
while (my $arg = shift @ARGV) {
    my ($key, $value) = split /=/, $arg;
    if (exists $args{$key}) {
        $args{$key} = $value;
    } else {
        push @pass_args, $arg;
    }
}
@ARGV = @pass_args;

# See lib/ExtUtils/MakeMaker.pm for details of how to influence
# the contents of the Makefile that is written.

my %opts=(
    'INSTALLDIRS'       => 'site',
    'NAME'		=> ucfirst($args{'name'}),
    'VERSION_FROM'	=> 'lib/Eidetic.pm', # finds $VERSION
    'ABSTRACT'     => 'Middleware lib for creating database applications in Perl', # retrieve abstract from module
    'AUTHOR'            => 'Bryce Harrington <bryce@osdl.org>, Kees Cook <kees@osdl.org>',
    'PREREQ_PM'         => {
   		'File::Temp'     => 0.12,
                'CGI'            => 2.56,
                'IO::File'       => 1.08,
                'POSIX'          => 0,
                'Digest::MD5'    => 2.09,
                'Text::Wrap'     => 2001.0929,
		'Test::More'     => 0.45,
                'File::Copy'     => 0,
                'File::Basename' => 0,
                'File::Spec'     => 0,
                'Template'       => 2.08,
         	'Sys::Hostname'  => 0,
		'Carp'           => 0,
		'Sort::Tree'     => 1.07,
		'Config::Simple' => 4.1,
                'DBI'            => 1.03,
		'Mail::Sender'   => 0.8,

#                'CGI::Pretty'   => 1.05,
#                'IO::File'      => 1.09,
#                'Fcntl'         => 1.04,
#                'Image::Magick' => 5.47,
#                'Date::Calc'    => 5.3

                           },
);

if ($ExtUtils::MakeMaker::VERSION > 5.45) {
	$opts{'PREREQ_FATAL'} = 1,
}

# This puts us in the site_perl directory, not dependant on any version
# of perl.
if (defined($Config{'sitelib_stem'}) && $Config{'sitelib_stem'} ne "") {
	print "stem is: $Config{'sitelib_stem'}\n";
	$opts{'INSTALLSITELIB'} = "";
	$opts{'INSTALLSITELIB'} = $args{'DESTDIR'}
		if ($] >= 5.008 ||
		    $ExtUtils::MakeMaker::VERSION =~ /5\.9[1-6]|6\.0[0-5]/);
	$opts{'INSTALLSITELIB'} .= $Config{'sitelib_stem'};
}

WriteMakefile(%opts);

sub install_clause {
    my ($source, $target_dir, $overwrite) = @_;
    $overwrite = 1 unless (defined($overwrite));

    die  "Source directory $source does not exist" unless (-e $source);

    my $text = '';

    $text .= "install :: $source\n";
    $text .= "\t\$(MKPATH) $target_dir\n";
    if ($overwrite) {
        $text .= "\t\$(RSYNC) -Cr $source $target_dir\n";
    } else {
        # Don't over-write
	# (This assumes we're installing a _file_, not a directory tree)
        my $dest_file = catfile( $target_dir, basename( $source ) );
	$text .= "\tif [ -f $dest_file ]; ";
	$text .= "then \$(CP) $source $dest_file.dist; ";
	$text .= "else \$(CP) $source $dest_file; fi\n";
    }
    $text .= "\t\$(CHMOD) -R a+rx $target_dir\n";
    $text .= "\n";
    return $text;
}


sub MY::postamble {
    my $self = shift;
    my $text = '';
    my $etcdir = '';

    $text .= "RSYNC = rsync\n";
    $text .= "CHOWN = chown\n\n";

    if (defined $self->{PREFIX} &&
        $self->{PREFIX} ne '' &&
        $self->{PREFIX} ne "/usr" &&
        $self->{PREFIX} ne "/usr/local" &&
        !defined($args{DESTDIR})) {
        die <<EOM;
If you're installing this anywhere other than "/usr" or "/usr/local", you
must specify the "DESTDIR" option on the "make" command line, otherwise
this script can't make sense of Perl's PREFIX setting.  It looks like your
PREFIX setting is '$self->{PREFIX}'.
EOM
    }

    my $destdir = $args{DESTDIR} || rootdir();
    my $prefix  = $self->{PREFIX};

    if ($prefix !~ /^$destdir/) {
    	$prefix = "$destdir$prefix";
    }

    my $real_etcdir = catdir( rootdir(), 'etc', $args{pkg_name} );
    $etcdir         = catdir( $destdir,  'etc', $args{pkg_name} );

    my $etcfiles = catfile( "etc", "$args{name}.conf" );
    $text .= install_clause($etcfiles, $etcdir, 0);

    # Substitute correct location of config file into script
    $text .= "# real_etcdir: $real_etcdir\n";
    $text .= "all :: \$(INST_LIB)/Eidetic.pm\n";
    $text .= "\techo Setting config path location in Eidetic.pm\n";
    $text .= qq|\t\$(PERL) -pi -e 's#\\\%ETCDIR\\\%#$real_etcdir#g' \$(INST_LIB)/Eidetic.pm|;
    $text .= "\n\n";

    # Notes to the user about next steps:
    $text .= "\t#\n\t# To purge & reinstall the db, do `make install_mysql_db`.\n";

    # Install database
    $text .= "\n\ninstall_mysql_db:\n";
    $text .= "\tcd sql && ./load_db.sh\n";
    $text .= "\n";

    return $text;
}



# vi:set ai ts=4 sw=4 expandtab:
