#!/usr/bin/perl -w
use strict;
use Eidetic;

my $name = 'eidetic.cgi';

my $e=new Eidetic(program_name => $name, trace => 1);

if (!$e->configure()) {
	die "Cannot configure program: $name\n";
}

# Default commands
my @stdcmds = qw(
               store
               search
               edit
               remove
               display
               );



$e->registerModule( name => 'office',
    table     => 'office',
    commands   => [ qw{
	       display  
	       search   
	       } ]
);

$e->registerModule( name => 'office_types',
    table     => 'office_type',
    commands   => [ qw{
	       display  
	       search   
	       } ]
);

$e->registerModule( name => 'components',
    table     => 'component',
    commands   => [ qw{
	       display  
	       search   
	       } ]
);

$e->registerModule( name => 'component_types',
    table     => 'component_type',
    commands   => [ qw{
	       display  
	       search   
	       } ]
);

$e->registerModule( name => 'hosts',
    table     => 'host',
    commands   => [ qw{
	       display  
	       search   
	       } ],
    queries => {
                   # This query allows putting a subtable of recent test
                   # requests onto the single host display page.  
                   host_test_requests =>    ['test_request',
                                             '1=1',
                                             'last_updated_date DESC'],
	},
);

$e->registerModule( name => 'host_types',
    table     => 'host_type',
    commands   => [ qw{
	       display  
	       search   
	       } ]
);

$e->registerModule( name => 'host_states',
    table     => 'host_state',
    commands   => [ qw{
	       display  
	       } ]
);

$e->registerModule( name => 'distro_tags',
    table => 'distro_tag',
    commands   => [ qw{
	       display  
	       } ]
);

$e->registerModule( name => 'kernel_tags',
    table => 'kernel_tag',
    commands   => [ qw{
	       display  
	       } ]

);

$e->registerModule( name => 'test_campaigns',
    table => 'test_campaign',
    commands   => [ qw{
	       display  
	       } ]

);

$e->registerModule( name => 'tests',
    table => 'test',
    commands   => [ qw{
	       display  
	       } ]

);

$e->registerModule( name => 'test_priorities',
    table => 'test_priority',
    commands   => [ qw{
	       display  
	       } ]

);


$e->registerModule( name => 'stp',
    table     => 'test_request',
    commands   => [ qw{
	display  
	search  
      } ],
    queries => {
                           stp =>    ['test_request', '', ''],
	},
);

$e->registerModule( name => 'stp_log',
    table => 'stp_log',
    commands   => \@stdcmds
);

$e->registerModule( name => 'patch_tag',
    table => 'patch_tag',
    commands   => \@stdcmds
);

$e->registerModule( name => 'cpu_types',
    table => 'cpu_type',
    commands   => [ qw{
	       display  
	       } ]

);

# STP test requests
$e->registerModule( name => 'test_requests',
    table => 'test_request',
    commands   => [ qw{
	display 
	search  
	} ],
    queries => {
                   all_queued_tests =>      ['test_request',
                                             'Status="queued"'
                                             .'&&Auto_publish=1',
                                             $e->getID().' DESC'],
                   all_in_progress_tests => ['test_request',
                                             'Status="running"'
                                             .'&&Auto_publish=1',
                                             $e->getID().' DESC'],
                   all_completed_tests =>   ['test_request',
                                             'Status="completed"'
                                             .'&&Auto_publish=1',
                                             $e->getID().' DESC'],
                   queued_tests =>          ['test_request',
                                             'Status="queued"'
                                             . '&&created_by=[%user_id%]',
                                             $e->getID().' DESC'],
                   in_progress_tests =>     ['test_request',
                                             'Status="running"'
                                             . '&&created_by=[%user_id%]',
                                             $e->getID().' DESC'],
                   completed_tests =>       ['test_request',
                                             'Status="completed"'
                                             . '&&created_by=[%user_id%]',
                                             $e->getID().' DESC'],
                   my_tests =>              ['test_request',
                                             'created_by=[%user_id%]',
                                             $e->getID().' DESC'],

	},
);

# Staff list in user table
$e->registerModule( name => 'users',
    table => 'user',
    commands   => [ qw{
	edit 
	reedit 
	store 
	thanks 
	} ],
    queries => {
                   staff =>                 ['user',
                                             'Real_email LIKE "%osdl.org"',
                                             'descriptor'],
                   # Module users & documents.  This allows a subtable
                   # to be placed on the user's page showing documents
                   # they own.
                   user_docs =>             ['document',
                                             '1=1',
                                             'created_date DESC'],

	},
);
 

# This is for putting news items onto the STP userpage.  It is
# the only remaining use of the articles module; the rest have
# been moved to the website.
$e->registerModule( name => 'articles',
    table => 'article',
    commands   => [ qw{
	       display  
	       } ],
    queries => {
                   linux_stp_news =>        ['article',
                                             "article_type_uid=7&&Topic='linux_stp'",
                                             'created_date DESC'],
	},

);

$e->registerModule( name => 'documents',
    table => 'document',
    commands   => [ qw{
               search   
               display  
               } ]

);

$e->registerModule( name => 'article_types',
    table => 'article_type',
    commands   => [ qw{
	       display  
	       } ]

);

$e->registerModule( name => 'document_types',
    table => 'document_type',
    commands   => [ qw{
               display  
               } ]

);


$e->registerModule( name => 'hardware_schedule',
    table => 'hardware_schedule',
    commands   => [ qw{
	       display  
	       } ]

);

$e->registerModule( name => 'event_types',
    table => 'event_type',
    commands   => [ qw{
	       display  
	       } ]

);

$e->registerModule( name => 'projects',
    table => 'project',
    commands   => [ qw{
	       store    
	       reedit   
	       display  
	       } ],
    queries => {
                   projects =>     ['project', '', ''],

                   # Module projects & documents.  Puts a subtable on the
                   # single project display page listing the project's
                   # documents.  This is replaced by the project_folders query.
                   project_docs =>          ['document',
                                             '1=1',
                                             'created_date DESC'],
                   # Module projects & test_requests.  This query allows putting
                   # a subtable listing a project's recent test requests onto
                   # the single project display page.
                   project_test_requests => ['test_request',
                                             '1=1',
                                             'last_updated_date DESC'],
                   # Module projects & folders.  This query allows putting a
                   # subtable listing a project's folders onto the single project
                   # display page.  It's how we're linking docsys into the project
                   # system.  Docsys 0.23 provides an entirely different way of 
                   # handling folders.
                   project_folders =>       ['folder',
                                             '1=1',
                                             'created_date DESC'],

                   # Module projects.  These queries are for generating various
                   # views into the lab projects.  This is part of the external
                   # project system.
                   proposed_projects =>     ['project',
                                             '(Acceptance_state="Proposal" OR Acceptance_state="Evaluation")',
                                             'Proj_name'],
                   active_projects =>       ['project',
                                             'Acceptance_state="Approved"',
                                             'Proj_name'],
                   completed_projects =>    ['project',
                                             'Acceptance_state="Closed" AND Development_state <> "Cancelled"',
                                             'Proj_name'],
                   proposed_projects_us =>   ['project',
                                             'Lab_uid=1 AND (Acceptance_state="Proposal" OR Acceptance_state="Evaluation")',
                                             'Proj_name'],
                   active_projects_us =>     ['project',
                                             'Lab_uid=1 AND Acceptance_state="Approved"',
                                             'Proj_name'],
                   completed_projects_us =>  ['project',
                                             'Lab_uid=1 AND Acceptance_state="Closed" AND Development_state <> "Cancelled"',
                                             'Proj_name'],
                   proposed_projects_jp =>  ['project',
                                             'Lab_uid=2 AND (Acceptance_state="Proposal" OR Acceptance_state="Evaluation")',
                                             'Proj_name'],
                   active_projects_jp =>    ['project',
                                             'Lab_uid=2 AND Acceptance_state="Approved"',
                                             'Proj_name'],
                   completed_projects_jp => ['project',
                                             'Lab_uid=2 AND Acceptance_state="Closed" AND Development_state <> "Cancelled"',
                                             'Proj_name'],



	},
);

$e->registerModule( name => 'campaign',
    table => 'campaign',
    commands   => [ qw{
	       store    
	       display  
	       } ]
);

$e->registerModule( name => 'archive_types',
    table => 'archive_type',
    commands   => [ qw{
	       display  
	       } ]
);

$e->registerModule( name => 'archive_statuss',
    table => 'archive_status',
    commands   => [ qw{
	       display  
	       } ]
);


$e->registerModule( name => 'lab',
    table => 'lab',
    commands   => [ qw{
	       display  
	       } ],
    queries => {
                   # Module labs and corps.  This is part of the internal
                   # document management system, allowing a list of the
                   # labs to be displayed as a subtable on the OSDL corporate page.
                   corp_labs =>             ['lab',
                                             '1=1',
                                             'descriptor'],
                   # Module labs and depts.  This is part of the internal
                   # document management system, allowing a list of the
                   # departments to be shown as a subtable on a given lab page.
                   lab_depts =>             ['dept',
                                             '1=1',
                                             'descriptor'],


	},
);

$e->registerModule( name => 'assets',
    table => 'asset',
    commands   => [ qw{
	       display  
	       } ]
);

$e->registerModule( name => 'archive',
    table => 'archive',
    commands   => [ qw{
	       display  
	       edit     
	       reedit   
	       search   
	       store    
	       upload   
	       } ]
);

$e->registerModule( name => 'folders',
    table => 'folder',
    commands   => [ qw{
               display  
               } ]


);

############
### MAIN ###
############

$e->main();

