# Before `make install' is performed this script should be runnable with
# `make test'. After `make install' it should work as `perl 1.t'

#########################

# change 'tests => 1' to 'tests => last_test_to_print';

use lib './t';
use Test::More;
my $Num_Tests = 28;
our $template;
require "testinfo.pm";
if ($template == 1) {
        plan tests => $Num_Tests;

	use_ok('Eidetic'); # test
	use_ok('Eidetic::Database'); # test
	use_ok('Eidetic::DBObject'); # test
	use_ok('Eidetic::Utility'); # test
}
else {
        plan skip_all => 'No Template::Toolkit';
}

###########
# Eidetic #
###########

# instantiate
my $e = new Eidetic;
isa_ok($e,"Eidetic"); 	# test

# verify all known functions
can_ok($e,qw(
        new
        configure
        getConfig
        useObject
        registerModule
        dispatch
        sendto_screen
        authorizeUser
        getFieldText
        do_userpage
        do_redirect
        do_search
        do_display
        do_edit
        validate
        do_store
        makeWidget
        processPage
        do_single
        do_many
        make_tree
        do_tree
        get_owner
        do_move
        do_attach
        do_detach
        do_remove_confirm
        do_remove
        parseTemplate
        sendto_email
        notifyStaff
        handleError
        substituteValues
        do_reset_password
        do_edit_password
        do_store_password
        do_users_thanks
	registerQuery
        getQuery
        makeHtmlTable
        genericHtmlForm
        collectData
        cgiCmd
        validateModule
        authenticate_user
        main
)); # test

is($e->{_GLOBALS}->{process_id},$$,"Process ID"); # test
is($e->{_GLOBALS}->{program_name},$0,"Default program name"); # test
undef $e;

# Do it again with arguments
my $testname="tester 1 2 3";
$e = new Eidetic( program_name => $testname );
isa_ok($e,"Eidetic"); # test
is($e->{_GLOBALS}->{program_name},$testname,"Passed program name"); # test
isnt($e->{_GLOBALS}->{program_name},$0,"Passed program name"); # test
undef $e;



#####################
# Eidetic::Database #
#####################
# instantiate
my $db = new Eidetic::Database(
	name => 'EIDETIC',
	host => 'mysql',
	user => 'statistics',
	pass => '',
);
isa_ok($db,"Eidetic::Database"); 	# test
# TODO: test new with bad values once it does not "confess"
can_ok($db,qw(
        new
        getfrom_db
        getnext_db
        createDatabaseTable
        addDatabaseFields
        DESTROY
        checkMissingElements
        checkInvalidElements
        insertRecord
        changePermission
        isInDatabase
        quickQuery
        getId
        selectRecords
        getDatabases
        getTables
        checkTableExistance
        getTableInfo
        checkType
        formatItem
        validateAndStoreRecord
        updateJunctions
        updateRecord
        hashedQuery
        affectedQuery
        insertID
)); # test

#####################
# Eidetic::DBObject #
#####################
# instantiate
my $obj = new Eidetic::DBObject($db, "user");
isa_ok($obj,"Eidetic::DBObject"); 	# test
# TODO: test new with bad values once it does not "confess"
can_ok($obj,qw(
        new
        attach
        get
        set
        fields
        lookForUIDs
)); # test

####################
# Eidetic::Utility #
####################
# TODO this is just a collection of functions, unfortunately

# trim
is(trim("  "),"","trim whitespace"); # test
is(trim(" asdf "),"asdf","trim word"); # test
is(trim(" as df "),"as df","trim words"); # test
is(trim("as df "),"as df","trim words trailing"); # test
is(trim(" as df"),"as df","trim words leading"); # test
is(trim(""),"","trim empty"); # test

# loadFile/sendto_file
my $goodfile="t/full-access";
my $badfile="t/no-access";
open(FILE,">$badfile") || die "$badfile: $!\n";
close(FILE);
chmod 0000, $badfile;

# sendto_file
# TODO how do we explicitly test open, write, and close failures?
SKIP: {
  skip "No easy way to test file access errors if you're 'root'", 1 if ($> == 0);
  eval { Eidetic::Utility::sendto_file($badfile); };
  like($@,qr/Failed to (write|open|close)/,"sendto_file fails"); # test
}
my $retval=0;
eval { $retval=Eidetic::Utility::sendto_file($goodfile,"title\n","body\n"); };
is($@,"","sendto_file stays happy"); # test
is($retval,1,"sendto_file returns 1"); # test

# loadFile
my $text="";
eval { $text = loadFile("t/dne"); };
like($@,qr/does not appear to exist/,"loadFile fails dns"); # test
# TODO: how do we test "cannot read" error?
SKIP: {
  skip "No easy way to test file access errors if you're 'root'", 1 if ($> == 0);
  eval { $text = loadFile($badfile); };
  like($@,qr/could not be opened/,"loadFile fails badfile"); # test
}
eval { $text = loadFile($goodfile); };
is($@,"","loadFile stays happy"); # test
like($text,qr/^title\nbody\n$/,"loadFile loads goodfile"); # test

unlink($goodfile);
unlink($badfile);


# TODO: test other utility functions as we discover they are actually used

#########################

# Insert your test code below, the Test::More module is use()ed here so read
# its man page ( perldoc Test::More ) for help writing this test script.

