# Before `make install' is performed this script should be runnable with
# `make test'. After `make install' it should work as `perl 1.t'

#########################

# change 'tests => 1' to 'tests => last_test_to_print';

use lib './t';
use Test::More;
my $Num_Tests = 37;
our $database;
require "testinfo.pm";
if ($database == 1) {
	plan tests => $Num_Tests;

	use_ok('Eidetic'); # test
}
else {
	plan skip_all => 'No database selected'
}

# instantiate
my $e = new Eidetic;
isa_ok($e,"Eidetic"); 	# test

# configure
$e->{_GLOBALS}->{program_name}=''; # hack to fail configure test
is($e->configure,  undef,"configure fails bad program_name Basename"); # test
like($e->{Error}, qr/^Could not determine app name from/,
	"correct error name"); # test

$e = new Eidetic;
isa_ok($e,"Eidetic"); 	# test
$e->{_GLOBALS}->{program_name}='missing'; # hack to fail configure test
is($e->configure,  undef,"configure fails missing conf file"); # test
like($e->{Error}, qr/^Config file not found: /,
	"correct error name"); # test

# TODO: once Eidetic/Database has been fixed to not die, test bad db cnxn

# TODO: figure out how to skip database test if there is no db available
$e = new Eidetic;
isa_ok($e,"Eidetic"); 	# test
$e->{_GLOBALS}->{program_name}="bad_defaults"; # bad defaults
is($e->configure( fields => {
			required => undef,
			optional => 5,
	}),  undef,"configure needs more config defaults"); # test
like($e->{Error}, qr/^Must define /,
	"correct error name"); # test

$e = new Eidetic;
isa_ok($e,"Eidetic"); 	# test
$e->{_GLOBALS}->{program_name}=$0; # regular default
is($e->configure( fields => {
			optional => 5,
	}),     1,"configure handles sane configuration"); # test

# getConfig
my $config=$e->getConfig;
is(defined($config),1,"getConfig returned"); # test
is($config->{optional},5,"configure passing defaults"); # test
is($config->{db_brand},"mysql","configure passing file settings"); # test
is(defined($config->{dne}),!1,"nothing extra from getConfig"); # test

# useObject
my $obj=11;
is($e->useObject($obj),$obj,"useObject takes a value"); # test

# registerModule
is($e->registerModule(),undef,"registerModule requires modulename"); # test
like($e->{Error}, qr/^No module name provided/,
	"correct error name"); # test
is($e->registerModule(name => "testing"),undef,
	"registerModule requires modulename"); # test
like($e->{Error}, qr/^No table name provided/,
	"correct error name"); # test
is($e->registerModule(name => "testing", table => "user", commands => ["this-cmd-dne"]),undef,
	"registerModule requires valid commands"); # test
like($e->{Error}, qr/^Undefined command/,
	"correct error name"); # test
is($e->registerModule(name => "testing", table => "user", commands => ["display","bad-bad-bad"]),undef,
	"registerModule requires valid commands"); # test
like($e->{Error}, qr/^Undefined command/,
	"correct error name"); # test
is($e->registerModule(name => "testing", table => "user", commands => ["display","search"]),1,
	"registerModule accepts valid commands"); # test
like($e->{Error}, qr/^$/,
	"empty error msg"); # test
# TODO: test for all stock Eidetic commands

# TODO: validateModule some more
is($e->validateModule("testing"),1,"module is valid"); # test


# registerQuery
$e->registerQuery( modulename => "testing", 
		   queries => {
				test1 => [ 'table1', 'filter1', 'sort1' ],
				test2 => [ 'table2', 'filter2', 'sort2' ],
			}
		);
my @test_array1=('table1','filter1','sort1');
my @test_array2=('table2','filter2','sort2');
my @ret_array;
@ret_array=$e->getQuery("testing","test1");
isnt(eq_array(\@test_array2,\@ret_array),1,"test2 query not stored"); # test
is(eq_array(\@test_array1,\@ret_array),1,"test1 query stored"); # test

@ret_array=$e->getQuery("testing","test2");
isnt(eq_array(\@test_array1,\@ret_array),1,"test1 query not stored"); # test
is(eq_array(\@test_array2,\@ret_array),1,"test2 query stored"); # test

$e->registerQuery(
		   queries => {
				test3 => [ 'table3', 'filter3', 'sort3' ],
			}
		);
my @test_array3=('table3','filter3','sort3');
@ret_array=$e->getQuery("testing","test3");
isnt(eq_array(\@test_array2,\@ret_array),1,"test2 query not stored"); # test
is(eq_array(\@test_array3,\@ret_array),1,"test3 query stored"); # test

$e->registerQuery( modulename => "testing", 
		   queries => {
				test3 => [ 'mytable3', 'myfilter3', 'mysort3' ],
			}
		);
my @my_test_array3=('mytable3','myfilter3','mysort3');
@ret_array=$e->getQuery("testing","test3");
isnt(eq_array(\@test_array3,\@ret_array),1,"global test3 query not stored"); # test
is(eq_array(\@my_test_array3,\@ret_array),1,"local test3 query stored"); # test

my @empty_array=('','','');
@ret_array=$e->getQuery("testing","dne");
is(eq_array(\@empty_array,\@ret_array),1,"empty on a dne query"); # test

#########################

# Insert your test code below, the Test::More module is use()ed here so read
# its man page ( perldoc Test::More ) for help writing this test script.

