# Before `make install' is performed this script should be runnable with
# `make test'. After `make install' it should work as `perl 1.t'

#########################

use lib './t';
use Test::More;
my $Num_Tests = 23;
our $database;
our $testuser;
our $testuid;
require "testinfo.pm";
if ($database == 1) {
        plan tests => $Num_Tests;

	use_ok('Eidetic'); # test
	use_ok('Eidetic::Database'); # test
	use_ok('Eidetic::DBObject'); # test
	use_ok('Eidetic::Utility'); # test
}
else {
        plan skip_all => 'No database selected';
}

###########
# Eidetic #
###########

my $tt2_output;

# Prepare environment for CGI processing
$ENV{'REMOTE_USER'}=$testuser;

# instantiate
my $e = new Eidetic;
isa_ok($e,"Eidetic"); 	# test
is($e->configure(
		tt2_config => {
			OUTPUT => \$tt2_output,
		},
	),1,"configure handles sane configuration"); # test
is($e->registerModule(name => "results_mod", table => "user", commands => ["display","search"]),1,
	"standard functions exist"); # test

# sendto_screen
eval { $e->sendto_screen({content => "The content"}); };
is($@,"","sendto_screen was happy"); # test
is($tt2_output,"header.tmpl\nTesting template\nThe content\nfooter.tmpl\n",
	"sendto_screen produced expected results"); # test
# TODO: how to test processing failures?
# TODO: test alternate header/footer processing


# authorizeUser
is($e->authorizeUser(),0,"requires args"); # test
is($e->authorizeUser("not-a-number"),0,"requires a number uid"); # test
is($e->authorizeUser($testuid+1),0,"requires correct uid"); # test
is($e->authorizeUser($testuid),1,"validates user"); # test

# TODO: test missing database
eval { $e->getFieldText(); };
like($@,qr/Undefined data record/,"requires data arg"); # test
eval { $e->getFieldText("data"); };
like($@,qr/Undefined source table/,"requires source arg"); # test
eval { $e->getFieldText("data","source"); };
like($@,qr/Undefined field/,"requires field arg"); # test
my $data={};
my $result="";
eval { $result=$e->getFieldText($data,"user","descriptor"); };
#like($@,qr/exist in table/,"getFieldText requires sane data arg"); # test
is($result,"err","getFieldText returns weird 'err' result"); # test

# TODO add this test once Database.pm does not use warnings
#$data={ "dne" => -1 };
#eval { $result=$e->getFieldText($data,"dne","dne"); };
#like($@,qr/Could not select records/,
#	"getFieldText needs sane table"); # test

$data={ "user_uid" => undef };
eval { $result=$e->getFieldText($data,"user","descriptor"); };
is($@,"","getFieldText can use blank 'source_uid'"); # test
is($result,'',"getFieldText returns blank"); # test

$data={ "user_uid" => $testuid };
eval { $result=$e->getFieldText($data,"user","descriptor"); };
is($@,"","getFieldText can use 'source_uid'"); # test
is($result,$testuser,"got correct field"); # test

$data={ "user" => $testuid };
eval { $result=$e->getFieldText($data,"user","descriptor"); };
is($@,"","getFieldText can use 'source_uid'"); # test
is($result,$testuser,"got correct field"); # test




#########################

# Insert your test code below, the Test::More module is use()ed here so read
# its man page ( perldoc Test::More ) for help writing this test script.

